# Generated by Django 4.1.7 on 2023-03-05 20:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('project_center', '0007_alter_company_options_alter_user_options_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
            ],
            options={
                'verbose_name': 'Activity',
                'verbose_name_plural': 'Activity',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('project_center.projectactivity',),
        ),
        migrations.AddField(
            model_name='projectactivity',
            name='reply',
            field=models.TextField(blank=True, null=True, verbose_name='Comments'),
        ),
        migrations.AlterField(
            model_name='project',
            name='users',
            field=models.ManyToManyField(blank=True, default=None, help_text='Users from the Company that are allowed to access the Project', null=True, to=settings.AUTH_USER_MODEL, verbose_name='Project Users'),
        ),
        migrations.AlterField(
            model_name='projectactivity',
            name='notes',
            field=models.TextField(blank=True, null=True, verbose_name='Activity Notes'),
        ),
        migrations.AlterField(
            model_name='projectactivity',
            name='project',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='project_center.project'),
        ),
    ]
