# Generated by Django 4.1.5 on 2023-02-16 02:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('project_center', '0004_projectcategory_display'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectstatus',
            name='display',
            field=models.BooleanField(default=True),
        ),
    ]
