# Generated by Django 4.1.7 on 2023-03-05 20:29

from django.db import migrations, models
import project_center.models


class Migration(migrations.Migration):

    dependencies = [
        ('project_center', '0008_activity_projectactivity_reply_alter_project_users_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='projectactivity',
            name='reply_file',
            field=models.FileField(blank=True, null=True, upload_to=project_center.models.activity_file_directory_path),
        ),
        migrations.AlterField(
            model_name='projectactivity',
            name='reply',
            field=models.TextField(blank=True, help_text='*** Adding or changing Comments will add a new Activity Record to the Project History', null=True, verbose_name='Comments'),
        ),
    ]
